from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.error import Error
from ..models.table_data_model import TableDataModel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Table")


@_attrs_define
class Table:
    """Dataview summary

    Attributes:
        description (str):
        errors (List['Error']):
        name (str):
        data_model (Union[Unset, TableDataModel]):
    """

    description: str
    errors: List["Error"]
    name: str
    data_model: Union[Unset, "TableDataModel"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        description = self.description
        errors = []
        for errors_item_data in self.errors:
            errors_item = errors_item_data.to_dict()
            errors.append(errors_item)

        name = self.name
        data_model: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_model, Unset):
            data_model = self.data_model.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "description": description,
                "errors": errors,
                "name": name,
            }
        )
        if data_model is not UNSET:
            field_dict["data_model"] = data_model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Table` from a dict"""
        d = src_dict.copy()
        description = d.pop("description")

        errors = []
        _errors = d.pop("errors")
        for errors_item_data in _errors:
            errors_item = Error.from_dict(errors_item_data)

            errors.append(errors_item)

        name = d.pop("name")

        _data_model = d.pop("data_model", UNSET)
        data_model: Union[Unset, TableDataModel]
        if isinstance(_data_model, Unset):
            data_model = UNSET
        else:
            data_model = TableDataModel.from_dict(_data_model)

        table = cls(
            description=description,
            errors=errors,
            name=name,
            data_model=data_model,
        )

        return table



"""Login and general Delfini operations"""
from collections.abc import Iterator
from urllib.parse import urlunparse

from .collections import DelfiniCollection
from .delfini_core import AuthenticatedClient as CoreAuthClient
from .delfini_core import Client as CoreClient
from .delfini_core import Login
from .delfini_core import Paginator
from .delfini_core.api.auth import auth_new_session
from .delfini_core.api.collections import collections_create_collection
from .delfini_core.api.collections import collections_get_collections
from .delfini_core.models import CollectionsGetCollectionsCollectionList
from .delfini_core.models import NewCollection
from .delfini_core.models import NewCollectionMetadata
from .delfini_core.types import UNSET


class DelfiniClient:
    """Main class for interacting with a Delfini instance."""

    def __init__(self, core: CoreAuthClient) -> None:
        self._client = core

    def __del__(self) -> None:
        if self._client._client:
            self._client._client.close()

    def get_collection_by_name(self, collection_name: str) -> DelfiniCollection:
        """Retrieve a single collection given its name.

        Raises:
            ValueError: if the collection could not be found or if more than one
                collection was found with the given name
        """
        collections = collections_get_collections.sync(
            client=self._client, name=collection_name
        )
        if len(collections.collections) == 1:
            return DelfiniCollection(collections.collections[0], self._client)

        raise ValueError(f"could not find collection: {collection_name}")

    def all_collections(self) -> Iterator[DelfiniCollection]:
        """Iterate over all collections visible to the user."""
        paginator = Paginator[CollectionsGetCollectionsCollectionList](
            collections_get_collections,
            self._client,
        )
        for page in paginator.paginate():
            for collection in page.collections:
                yield DelfiniCollection(collection, self._client)

    def new_collection(
        self, name: str, description: str, metadata: dict[str, str] = {}
    ) -> DelfiniCollection:
        """Create a new collection.

        Args:
            name: The short name of the collection
            description: A longer plain text description of the collection
            metadata: Optional key-value pairs of metadata on the collection

        """
        collection = collections_create_collection.sync(
            client=self._client,
            body=NewCollection(
                name=name,
                description=description,
                metadata=(
                    NewCollectionMetadata.from_dict(metadata) if metadata else UNSET
                ),
            ),
        )
        return DelfiniCollection(collection, self._client)


def make_base_url(host: str, scheme: str = "https") -> str:
    """Create a base URL for a Delfini instance.

    Args:
        host: The hostname of your Delfini instance, such as 'delfini.bioteam.net'.
        scheme: either 'https' or 'http'

    Returns:
        the Delfini API base URL

    """
    return urlunparse([scheme, host, "/api/v1", None, None, None])


def login(host: str, scheme: str = "https") -> DelfiniClient:
    """Login to a Delfini instance.

    This interactive login method will print a URL that you will need
    to visit in order to activate the login session. The function will
    wait to return until the session has successfully been activated.

    Args:
        host: The hostname of your Delfini instance, such as 'delfini.bioteam.net'.
        scheme: either 'https' or 'http'

    Returns:
        A :py:class:`DelfiniClient`

    """

    # get a session ID
    core = CoreClient(base_url=make_base_url(host, scheme))
    session_token = auth_new_session.sync(client=core)

    # construct the URL to activate the session
    activation_url = urlunparse(
        [
            scheme,
            host,
            f"/login/activate/{session_token.activation_code}",
            None,
            None,
            None,
        ]
    )

    print("To activate your session, visit the URL below:")
    print("  ", activation_url)
    print()
    print("Waiting for session activation...")

    # wait for the session to be activated
    authenticated = Login(core).with_session_id(session_token.session_id, wait=True)

    return DelfiniClient(authenticated)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.collection_authorization_remove_identity_action import (
    CollectionAuthorizationRemoveIdentityAction,
)


T = TypeVar("T", bound="CollectionAuthorizationRemoveIdentity")


@_attrs_define
class CollectionAuthorizationRemoveIdentity:
    """CollectionAuthorizationRemoveIdentity model

    Attributes:
        action (CollectionAuthorizationRemoveIdentityAction):
        identity (str):
    """

    action: CollectionAuthorizationRemoveIdentityAction
    identity: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        action = self.action.value
        identity = self.identity

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "action": action,
                "identity": identity,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionAuthorizationRemoveIdentity` from a dict"""
        d = src_dict.copy()
        action = CollectionAuthorizationRemoveIdentityAction(d.pop("action"))

        identity = d.pop("identity")

        collection_authorization_remove_identity = cls(
            action=action,
            identity=identity,
        )

        return collection_authorization_remove_identity



from enum import Enum


class CollectionsItemsCompleteMultipartUploadBodyAction(str, Enum):
    CANCEL = "cancel"
    COMPLETE = "complete"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.group_metadata import GroupMetadata
from ..models.visibility_level import VisibilityLevel


T = TypeVar("T", bound="Group")


@_attrs_define
class Group:
    """Group model

    Attributes:
        account_linked (bool):
        controlled_access (bool):
        created_on (datetime.datetime):
        id (str):
        metadata (GroupMetadata):
        name (str):
        visibility_level (VisibilityLevel):
    """

    account_linked: bool
    controlled_access: bool
    created_on: datetime.datetime
    id: str
    metadata: "GroupMetadata"
    name: str
    visibility_level: VisibilityLevel

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account_linked = self.account_linked
        controlled_access = self.controlled_access
        created_on = self.created_on.isoformat()
        id = self.id
        metadata = self.metadata.to_dict()
        name = self.name
        visibility_level = self.visibility_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accountLinked": account_linked,
                "controlledAccess": controlled_access,
                "createdOn": created_on,
                "id": id,
                "metadata": metadata,
                "name": name,
                "visibilityLevel": visibility_level,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Group` from a dict"""
        d = src_dict.copy()
        account_linked = d.pop("accountLinked")

        controlled_access = d.pop("controlledAccess")

        created_on = isoparse(d.pop("createdOn"))

        id = d.pop("id")

        metadata = GroupMetadata.from_dict(d.pop("metadata"))

        name = d.pop("name")

        visibility_level = VisibilityLevel(d.pop("visibilityLevel"))

        group = cls(
            account_linked=account_linked,
            controlled_access=controlled_access,
            created_on=created_on,
            id=id,
            metadata=metadata,
            name=name,
            visibility_level=visibility_level,
        )

        return group



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_column import ItemColumn


T = TypeVar("T", bound="QueryDataElementsFramesItem")


@_attrs_define
class QueryDataElementsFramesItem:
    """QueryDataElementsFramesItem model

    Attributes:
        columns (Union[List['ItemColumn'], None]):
        step_name (str):
    """

    columns: Union[List["ItemColumn"], None]
    step_name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        columns: Union[List[Dict[str, Any]], None]
        if isinstance(self.columns, list):
            columns = []
            for componentsschemasitem_columns_type_0_item_data in self.columns:
                componentsschemasitem_columns_type_0_item = (
                    componentsschemasitem_columns_type_0_item_data.to_dict()
                )
                columns.append(componentsschemasitem_columns_type_0_item)

        else:
            columns = self.columns
        step_name = self.step_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "columns": columns,
                "step_name": step_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`QueryDataElementsFramesItem` from a dict"""
        d = src_dict.copy()

        def _parse_columns(data: object) -> Union[List["ItemColumn"], None]:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasitem_columns_type_0 = []
                _componentsschemasitem_columns_type_0 = data
                for (
                    componentsschemasitem_columns_type_0_item_data
                ) in _componentsschemasitem_columns_type_0:
                    componentsschemasitem_columns_type_0_item = ItemColumn.from_dict(
                        componentsschemasitem_columns_type_0_item_data
                    )

                    componentsschemasitem_columns_type_0.append(
                        componentsschemasitem_columns_type_0_item
                    )

                return componentsschemasitem_columns_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["ItemColumn"], None], data)

        columns = _parse_columns(d.pop("columns"))

        step_name = d.pop("step_name")

        query_data_elements_frames_item = cls(
            columns=columns,
            step_name=step_name,
        )

        return query_data_elements_frames_item



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element_permissible_values_value_set_value_set_item import (
    DataElementPermissibleValuesValueSetValueSetItem,
)


T = TypeVar("T", bound="DataElementPermissibleValuesValueSet")


@_attrs_define
class DataElementPermissibleValuesValueSet:
    """DataElementPermissibleValuesValueSet model

    Attributes:
        value_set (List['DataElementPermissibleValuesValueSetValueSetItem']): One or more values that exactly match the
            data.
    """

    value_set: List["DataElementPermissibleValuesValueSetValueSetItem"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        value_set = []
        for value_set_item_data in self.value_set:
            value_set_item = value_set_item_data.to_dict()
            value_set.append(value_set_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "valueSet": value_set,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesValueSet` from a dict"""
        d = src_dict.copy()
        value_set = []
        _value_set = d.pop("valueSet")
        for value_set_item_data in _value_set:
            value_set_item = DataElementPermissibleValuesValueSetValueSetItem.from_dict(
                value_set_item_data
            )

            value_set.append(value_set_item)

        data_element_permissible_values_value_set = cls(
            value_set=value_set,
        )

        return data_element_permissible_values_value_set



import argparse
import json
import os.path
import sys
from getpass import getpass
from typing import Optional
from urllib.parse import urlparse

from pydelfini.client import DelfiniClient
from pydelfini.client import login as interactive_login
from pydelfini.client import make_base_url
from pydelfini.delfini_core import Client as CoreClient
from pydelfini.delfini_core.errors import UnexpectedStatus
from pydelfini.delfini_core.login import Login
from pydelfini.delfini_core.login import LoginError
from pydelfini.delfini_core.login import to_token_file
from pydelfini.delfini_core.login import token_file_get_base_url

from .commands import Commands


def login(base_url: str, token_file: str, username: Optional[str]) -> DelfiniClient:
    core = CoreClient(base_url=base_url)

    # first try to log in with the token file
    if os.path.exists(token_file):
        try:
            authenticated = Login(core).from_token_file(token_file)
            return DelfiniClient(authenticated)
        except LoginError:
            print("Previous session is invalid...")

    # that didn't work, if the username is provided, perform a
    # semi-interactive login
    if username:
        password = getpass()
        authenticated = Login(core).with_password(username, password)
        return DelfiniClient(authenticated)

    # perform a full interactive login
    parsed_url = urlparse(base_url)
    return interactive_login(parsed_url.netloc, parsed_url.scheme)


def main(argv: Optional[list[str]] = None) -> None:
    argparser = argparse.ArgumentParser(
        prog="dcmd",
    )
    argparser.add_argument(
        "-H", "--hostname", metavar="HOSTNAME", help="connect to HOSTNAME"
    )
    argparser.add_argument(
        "--insecure",
        dest="scheme",
        default="https",
        action="store_const",
        const="http",
        help="connect using http rather than https",
    )
    argparser.add_argument(
        "-k",
        "--token-file",
        default="~/.dcmd-token",
        metavar="FN",
        help="use FN as the file to hold the login token",
    )
    argparser.add_argument(
        "-u",
        "--username",
        metavar="USERNAME",
        help="disable interactive login; log in with USERNAME",
    )
    argparser.add_argument(
        "-O",
        "--output-type",
        default="auto",
        choices=["auto", "yaml", "json", "table"],
        help="set output type",
    )
    Commands._add_to_parser(argparser.add_subparsers(title="commands", required=True))

    args = argparser.parse_args(argv if argv is not None else sys.argv[1:])

    token_file = os.path.expanduser(args.token_file)
    if os.path.exists(token_file) and not args.hostname:
        base_url = token_file_get_base_url(token_file)
    elif args.hostname:
        base_url = make_base_url(args.hostname, args.scheme)
    else:
        print(
            "No hostname specified! Use the `-H` option to specify the name of your"
            " Delfini instance."
        )
        sys.exit(1)

    client = login(base_url, token_file, args.username)
    if token_file:
        to_token_file(client._client, token_file)

    if hasattr(args, "func"):
        try:
            args.func(args.cls(client, args))
        except UnexpectedStatus as ex:
            if argv is not None:
                raise

            print(str(ex))
            try:
                parsed = json.loads(ex.content)
                print("error:", parsed["error"])
                if parsed["detail"]:
                    print("detail:", parsed["detail"])
            except (json.JSONDecodeError, KeyError):
                print(ex.content)
            sys.exit(1)

    else:
        argparser.print_usage()



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.admin_grants import AdminGrants
from ..models.operations import Operations
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="AdminList")


@_attrs_define
class AdminList:
    """AdminList model

    Attributes:
        grants (List['AdminGrants']):
        operations (Union[Unset, List[Operations]]):
    """

    grants: List["AdminGrants"]
    operations: Union[Unset, List[Operations]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        grants = []
        for grants_item_data in self.grants:
            grants_item = grants_item_data.to_dict()
            grants.append(grants_item)

        operations: Union[Unset, List[str]] = UNSET
        if not isinstance(self.operations, Unset):
            operations = []
            for operations_item_data in self.operations:
                operations_item = operations_item_data.value
                operations.append(operations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "grants": grants,
            }
        )
        if operations is not UNSET:
            field_dict["operations"] = operations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AdminList` from a dict"""
        d = src_dict.copy()
        grants = []
        _grants = d.pop("grants")
        for grants_item_data in _grants:
            grants_item = AdminGrants.from_dict(grants_item_data)

            grants.append(grants_item)

        operations = []
        _operations = d.pop("operations", UNSET)
        for operations_item_data in _operations or []:
            operations_item = Operations(operations_item_data)

            operations.append(operations_item)

        admin_list = cls(
            grants=grants,
            operations=operations,
        )

        return admin_list



""" Contains all the data models used in inputs/outputs """
from .account import Account
from .account_create_account_body import AccountCreateAccountBody
from .account_create_account_body_metadata import AccountCreateAccountBodyMetadata
from .account_list import AccountList
from .account_list_accounts_visibility_level import AccountListAccountsVisibilityLevel
from .account_metadata import AccountMetadata
from .account_update_account_body import AccountUpdateAccountBody
from .account_update_account_body_metadata import AccountUpdateAccountBodyMetadata
from .admin_grant_admin_admin_list_request import AdminGrantAdminAdminListRequest
from .admin_grants import AdminGrants
from .admin_list import AdminList
from .auth_activate_session_body import AuthActivateSessionBody
from .auth_authenticate_user_authentication_request import (
    AuthAuthenticateUserAuthenticationRequest,
)
from .auth_authenticate_user_response_200 import AuthAuthenticateUserResponse200
from .auth_get_csrf_response_200 import AuthGetCsrfResponse200
from .auth_get_session_response_200 import AuthGetSessionResponse200
from .auth_provider_authorized_provider import AuthProviderAuthorizedProvider
from .auth_provider_login_provider import AuthProviderLoginProvider
from .auth_provider_signin_provider_provider import AuthProviderSigninProviderProvider
from .auth_provider_signin_provider_response_200 import (
    AuthProviderSigninProviderResponse200,
)
from .auth_providers_req_response_200 import AuthProvidersReqResponse200
from .auth_signout_no_data import AuthSignoutNoData
from .authorized_actions_item import AuthorizedActionsItem
from .bundle import Bundle
from .bundle_definition import BundleDefinition
from .cache_file import CacheFile
from .cache_file_type import CacheFileType
from .cache_mem import CacheMem
from .cache_mem_type import CacheMemType
from .cdes_list_data_elements_response_200 import CdesListDataElementsResponse200
from .cdes_new_cdeset_body import CdesNewCdesetBody
from .cdeset import Cdeset
from .cdeset_list import CdesetList
from .collection import Collection
from .collection_access_level import CollectionAccessLevel
from .collection_authorization import CollectionAuthorization
from .collection_authorization_change import CollectionAuthorizationChange
from .collection_authorization_grant import CollectionAuthorizationGrant
from .collection_authorization_inherited import CollectionAuthorizationInherited
from .collection_authorization_permit import CollectionAuthorizationPermit
from .collection_authorization_remove_identity import (
    CollectionAuthorizationRemoveIdentity,
)
from .collection_authorization_remove_identity_action import (
    CollectionAuthorizationRemoveIdentityAction,
)
from .collection_authorization_set_identity import CollectionAuthorizationSetIdentity
from .collection_authorization_set_identity_action import (
    CollectionAuthorizationSetIdentityAction,
)
from .collection_authorization_update_access_level import (
    CollectionAuthorizationUpdateAccessLevel,
)
from .collection_authorization_update_access_level_action import (
    CollectionAuthorizationUpdateAccessLevelAction,
)
from .collection_cde_stats import CollectionCdeStats
from .collection_cde_stats_item_distrib_item import CollectionCdeStatsItemDistribItem
from .collection_cde_stats_per_cde import CollectionCdeStatsPerCde
from .collection_metadata import CollectionMetadata
from .collection_move_datastore import CollectionMoveDatastore
from .collection_role import CollectionRole
from .collection_stats import CollectionStats
from .collection_stats_item_stats import CollectionStatsItemStats
from .collection_stats_item_stats_additional_property import (
    CollectionStatsItemStatsAdditionalProperty,
)
from .collection_stats_item_stats_additional_property_num_failed import (
    CollectionStatsItemStatsAdditionalPropertyNumFailed,
)
from .collections_dictionaries_copy_to_cdeset_body import (
    CollectionsDictionariesCopyToCdesetBody,
)
from .collections_dictionaries_list_bundles_response_200 import (
    CollectionsDictionariesListBundlesResponse200,
)
from .collections_dictionaries_list_data_element_refs_response_200 import (
    CollectionsDictionariesListDataElementRefsResponse200,
)
from .collections_dictionaries_list_data_elements_response_200 import (
    CollectionsDictionariesListDataElementsResponse200,
)
from .collections_dictionaries_list_dictionaries_response_200 import (
    CollectionsDictionariesListDictionariesResponse200,
)
from .collections_get_collection_apc_members_filter import (
    CollectionsGetCollectionApcMembersFilter,
)
from .collections_get_collection_authorization_filter import (
    CollectionsGetCollectionAuthorizationFilter,
)
from .collections_get_collections_collection_list import (
    CollectionsGetCollectionsCollectionList,
)
from .collections_get_collections_sort import CollectionsGetCollectionsSort
from .collections_get_collections_sort_dir import CollectionsGetCollectionsSortDir
from .collections_get_collections_version import CollectionsGetCollectionsVersion
from .collections_items_complete_multipart_upload_body import (
    CollectionsItemsCompleteMultipartUploadBody,
)
from .collections_items_complete_multipart_upload_body_action import (
    CollectionsItemsCompleteMultipartUploadBodyAction,
)
from .collections_items_complete_multipart_upload_body_checksum import (
    CollectionsItemsCompleteMultipartUploadBodyChecksum,
)
from .collections_items_complete_multipart_upload_body_parts_item import (
    CollectionsItemsCompleteMultipartUploadBodyPartsItem,
)
from .collections_items_copy_item_body import CollectionsItemsCopyItemBody
from .collections_items_create_item_files_form_style_upload import (
    CollectionsItemsCreateItemFilesFormStyleUpload,
)
from .collections_items_create_item_files_form_style_upload_metadata import (
    CollectionsItemsCreateItemFilesFormStyleUploadMetadata,
)
from .collections_items_create_item_json_new_item_request import (
    CollectionsItemsCreateItemJsonNewItemRequest,
)
from .collections_items_create_item_json_new_item_request_content_type_1 import (
    CollectionsItemsCreateItemJsonNewItemRequestContentType1,
)
from .collections_items_get_item_data_dl import CollectionsItemsGetItemDataDl
from .collections_items_initiate_multipart_upload_body import (
    CollectionsItemsInitiateMultipartUploadBody,
)
from .collections_items_list_items_response_200 import (
    CollectionsItemsListItemsResponse200,
)
from .collections_items_put_item_body import CollectionsItemsPutItemBody
from .collections_items_put_item_body_content import CollectionsItemsPutItemBodyContent
from .collections_items_put_item_body_content_checksum import (
    CollectionsItemsPutItemBodyContentChecksum,
)
from .collections_tables_get_formatted_table_data_export_format import (
    CollectionsTablesGetFormattedTableDataExportFormat,
)
from .collections_tables_get_table_data_elements_response_200 import (
    CollectionsTablesGetTableDataElementsResponse200,
)
from .collections_tables_get_table_data_elements_response_200_element_map import (
    CollectionsTablesGetTableDataElementsResponse200ElementMap,
)
from .collections_tables_get_table_data_elements_response_200_error_map import (
    CollectionsTablesGetTableDataElementsResponse200ErrorMap,
)
from .collections_tables_list_tables_response_200 import (
    CollectionsTablesListTablesResponse200,
)
from .collections_tables_preview_table_data_body import (
    CollectionsTablesPreviewTableDataBody,
)
from .column_schema_type import ColumnSchemaType
from .data_dictionary import DataDictionary
from .data_dictionary_source_item import DataDictionarySourceItem
from .data_element import DataElement
from .data_element_bundle import DataElementBundle
from .data_element_concept import DataElementConcept
from .data_element_concept_applies_to import DataElementConceptAppliesTo
from .data_element_data_type import DataElementDataType
from .data_element_definition import DataElementDefinition
from .data_element_definition_definition_type import DataElementDefinitionDefinitionType
from .data_element_permissible_values_date_time_format import (
    DataElementPermissibleValuesDateTimeFormat,
)
from .data_element_permissible_values_date_time_format_date_time_format import (
    DataElementPermissibleValuesDateTimeFormatDateTimeFormat,
)
from .data_element_permissible_values_date_time_format_date_time_format_format_type_0 import (
    DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0,
)
from .data_element_permissible_values_date_time_format_date_time_format_format_type_0_isoformat import (
    DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0Isoformat,
)
from .data_element_permissible_values_date_time_format_date_time_format_format_type_1 import (
    DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1,
)
from .data_element_permissible_values_external_reference import (
    DataElementPermissibleValuesExternalReference,
)
from .data_element_permissible_values_external_reference_external_reference import (
    DataElementPermissibleValuesExternalReferenceExternalReference,
)
from .data_element_permissible_values_number_range import (
    DataElementPermissibleValuesNumberRange,
)
from .data_element_permissible_values_number_range_number_range import (
    DataElementPermissibleValuesNumberRangeNumberRange,
)
from .data_element_permissible_values_text_range import (
    DataElementPermissibleValuesTextRange,
)
from .data_element_permissible_values_text_range_text_range import (
    DataElementPermissibleValuesTextRangeTextRange,
)
from .data_element_permissible_values_value_set import (
    DataElementPermissibleValuesValueSet,
)
from .data_element_permissible_values_value_set_value_set_item import (
    DataElementPermissibleValuesValueSetValueSetItem,
)
from .data_element_reference import DataElementReference
from .datastore import Datastore
from .datastore_type import DatastoreType
from .dictionary_search_options import DictionarySearchOptions
from .dictionary_search_options_options import DictionarySearchOptionsOptions
from .dictionary_search_options_options_additional_property import (
    DictionarySearchOptionsOptionsAdditionalProperty,
)
from .dictionary_search_options_options_additional_property_type import (
    DictionarySearchOptionsOptionsAdditionalPropertyType,
)
from .error import Error
from .event_metric import EventMetric
from .event_metric_dimensions import EventMetricDimensions
from .event_metric_type import EventMetricType
from .federation_activity import FederationActivity
from .federation_address_or_object_type_2 import FederationAddressOrObjectType2
from .federation_collection import FederationCollection
from .federation_collection_page import FederationCollectionPage
from .federation_collection_page_type import FederationCollectionPageType
from .federation_collection_type import FederationCollectionType
from .federation_user import FederationUser
from .federation_user_type import FederationUserType
from .group import Group
from .group_create_group_body import GroupCreateGroupBody
from .group_create_group_body_metadata import GroupCreateGroupBodyMetadata
from .group_get_groups_response_200 import GroupGetGroupsResponse200
from .group_get_groups_response_200_self_roles import GroupGetGroupsResponse200SelfRoles
from .group_get_groups_visibility_level import GroupGetGroupsVisibilityLevel
from .group_member import GroupMember
from .group_members import GroupMembers
from .group_members_users import GroupMembersUsers
from .group_metadata import GroupMetadata
from .group_role import GroupRole
from .group_update_group_body import GroupUpdateGroupBody
from .group_update_group_body_metadata import GroupUpdateGroupBodyMetadata
from .highlights import Highlights
from .highlights_metadata import HighlightsMetadata
from .identity import Identity
from .item import Item
from .item_column import ItemColumn
from .item_content import ItemContent
from .item_content_checksum import ItemContentChecksum
from .item_metadata import ItemMetadata
from .item_parser import ItemParser
from .item_parser_options import ItemParserOptions
from .item_sensitivity_labels import ItemSensitivityLabels
from .item_status import ItemStatus
from .item_status_detail import ItemStatusDetail
from .item_status_details import ItemStatusDetails
from .item_status_value import ItemStatusValue
from .item_storage import ItemStorage
from .item_storage_checksum import ItemStorageChecksum
from .item_type import ItemType
from .metadata_field import MetadataField
from .metadata_field_group import MetadataFieldGroup
from .metadata_field_group_resource import MetadataFieldGroupResource
from .metadata_field_groups import MetadataFieldGroups
from .metadata_field_groups_data_elements import MetadataFieldGroupsDataElements
from .metadata_fields import MetadataFields
from .metadata_fields_data_elements import MetadataFieldsDataElements
from .metadata_get_field_groups_by_resource_resource_type import (
    MetadataGetFieldGroupsByResourceResourceType,
)
from .metadata_metric import MetadataMetric
from .metadata_metric_type import MetadataMetricType
from .metadata_set_field_groups_body import MetadataSetFieldGroupsBody
from .metadata_tags import MetadataTags
from .metric_agg_func import MetricAggFunc
from .metric_data import MetricData
from .metric_event import MetricEvent
from .metric_event_list import MetricEventList
from .metric_list import MetricList
from .metric_log import MetricLog
from .metric_log_entry import MetricLogEntry
from .metric_log_entry_dimensions import MetricLogEntryDimensions
from .metric_timepoint import MetricTimepoint
from .metric_timepoint_partitions import MetricTimepointPartitions
from .metrics_export_event_data_format import MetricsExportEventDataFormat
from .metrics_query_event_data_period import MetricsQueryEventDataPeriod
from .multipart_upload_details import MultipartUploadDetails
from .multipart_upload_details_target_urls import MultipartUploadDetailsTargetUrls
from .new_collection import NewCollection
from .new_collection_metadata import NewCollectionMetadata
from .new_collection_version import NewCollectionVersion
from .new_collection_version_metadata import NewCollectionVersionMetadata
from .new_data_element import NewDataElement
from .new_data_element_data_type import NewDataElementDataType
from .new_data_element_ref import NewDataElementRef
from .new_project import NewProject
from .new_project_metadata import NewProjectMetadata
from .new_user import NewUser
from .oauth_provider import OauthProvider
from .operations import Operations
from .ordered_dictionary import OrderedDictionary
from .pagination import Pagination
from .parser import Parser
from .parser_option import ParserOption
from .parser_option_type import ParserOptionType
from .parser_options import ParserOptions
from .parsers_get_parsers_response_200 import ParsersGetParsersResponse200
from .parsers_resolve_prql_modules_response_200 import (
    ParsersResolvePrqlModulesResponse200,
)
from .preferred_order import PreferredOrder
from .project import Project
from .project_metadata import ProjectMetadata
from .projects_get_projects_project_list import ProjectsGetProjectsProjectList
from .prql_module import PrqlModule
from .query_data_element_request import QueryDataElementRequest
from .query_data_elements import QueryDataElements
from .query_data_elements_element_map import QueryDataElementsElementMap
from .query_data_elements_error_map import QueryDataElementsErrorMap
from .query_data_elements_frames_item import QueryDataElementsFramesItem
from .root_get_datastores_response_200 import RootGetDatastoresResponse200
from .search_accounts_response import SearchAccountsResponse
from .search_accounts_response_hits_item import SearchAccountsResponseHitsItem
from .search_accounts_response_hits_item_projects_item import (
    SearchAccountsResponseHitsItemProjectsItem,
)
from .search_collections_response import SearchCollectionsResponse
from .search_collections_response_hits_item import SearchCollectionsResponseHitsItem
from .search_collections_response_hits_item_data_elements_item import (
    SearchCollectionsResponseHitsItemDataElementsItem,
)
from .search_collections_response_hits_item_items_item import (
    SearchCollectionsResponseHitsItemItemsItem,
)
from .search_dictionaries_by_item_column_hits import SearchDictionariesByItemColumnHits
from .search_dictionaries_by_item_column_hits_search_dictionaries_hit import (
    SearchDictionariesByItemColumnHitsSearchDictionariesHit,
)
from .search_dictionaries_by_item_inverse_response import (
    SearchDictionariesByItemInverseResponse,
)
from .search_dictionaries_by_item_response import SearchDictionariesByItemResponse
from .search_dictionaries_inverse_result import SearchDictionariesInverseResult
from .search_dictionaries_inverse_result_queries_item import (
    SearchDictionariesInverseResultQueriesItem,
)
from .search_dictionaries_response import SearchDictionariesResponse
from .search_get_dictionary_search_options_response_200 import (
    SearchGetDictionarySearchOptionsResponse200,
)
from .search_search_accounts_body import SearchSearchAccountsBody
from .search_search_accounts_body_types_item import SearchSearchAccountsBodyTypesItem
from .search_search_collections_body import SearchSearchCollectionsBody
from .search_search_collections_body_types_item import (
    SearchSearchCollectionsBodyTypesItem,
)
from .search_search_dictionaries_body import SearchSearchDictionariesBody
from .search_search_dictionaries_body_options import SearchSearchDictionariesBodyOptions
from .search_search_dictionaries_by_item_body import SearchSearchDictionariesByItemBody
from .search_search_dictionaries_by_item_body_method import (
    SearchSearchDictionariesByItemBodyMethod,
)
from .search_search_dictionaries_by_item_inverse_body import (
    SearchSearchDictionariesByItemInverseBody,
)
from .search_search_dictionaries_by_item_inverse_body_method import (
    SearchSearchDictionariesByItemInverseBodyMethod,
)
from .server_error import ServerError
from .session_token import SessionToken
from .session_user import SessionUser
from .session_user_metadata import SessionUserMetadata
from .system_configuration import SystemConfiguration
from .system_configuration_authentication import SystemConfigurationAuthentication
from .system_configuration_authentication_credentials import (
    SystemConfigurationAuthenticationCredentials,
)
from .system_configuration_authentication_credentials_type import (
    SystemConfigurationAuthenticationCredentialsType,
)
from .system_configuration_authentication_github import (
    SystemConfigurationAuthenticationGithub,
)
from .system_configuration_authentication_github_type import (
    SystemConfigurationAuthenticationGithubType,
)
from .system_configuration_authentication_google import (
    SystemConfigurationAuthenticationGoogle,
)
from .system_configuration_authentication_google_type import (
    SystemConfigurationAuthenticationGoogleType,
)
from .system_configuration_authentication_oauth import (
    SystemConfigurationAuthenticationOauth,
)
from .system_configuration_authentication_oauth_type import (
    SystemConfigurationAuthenticationOauthType,
)
from .system_configuration_cache import SystemConfigurationCache
from .system_configuration_cache_cache_type import SystemConfigurationCacheCacheType
from .system_configuration_datastores import SystemConfigurationDatastores
from .system_configuration_datastores_local import SystemConfigurationDatastoresLocal
from .system_configuration_datastores_local_type import (
    SystemConfigurationDatastoresLocalType,
)
from .system_configuration_datastores_plugin import SystemConfigurationDatastoresPlugin
from .system_configuration_datastores_plugin_options import (
    SystemConfigurationDatastoresPluginOptions,
)
from .system_configuration_datastores_plugin_type import (
    SystemConfigurationDatastoresPluginType,
)
from .system_configuration_datastores_s3 import SystemConfigurationDatastoresS3
from .system_configuration_datastores_s3_type import SystemConfigurationDatastoresS3Type
from .system_configuration_plugins import SystemConfigurationPlugins
from .system_configuration_plugins_additional_property import (
    SystemConfigurationPluginsAdditionalProperty,
)
from .system_configuration_sessions import SystemConfigurationSessions
from .system_configuration_workers import SystemConfigurationWorkers
from .system_configuration_workers_backend_type import (
    SystemConfigurationWorkersBackendType,
)
from .table import Table
from .table_data import TableData
from .table_data_data_item import TableDataDataItem
from .table_data_data_model import TableDataDataModel
from .table_data_model import TableDataModel
from .task_def import TaskDef
from .task_def_data_type_4 import TaskDefDataType4
from .task_def_retry_item import TaskDefRetryItem
from .task_result import TaskResult
from .task_result_data_type_4 import TaskResultDataType4
from .task_result_errors_item import TaskResultErrorsItem
from .task_result_status import TaskResultStatus
from .task_schedule_def import TaskScheduleDef
from .task_schedule_def_frequency import TaskScheduleDefFrequency
from .tasks_results import TasksResults
from .tasks_schedules import TasksSchedules
from .tasks_schedules_schedules_item import TasksSchedulesSchedulesItem
from .tasks_stats import TasksStats
from .termset import Termset
from .termset_additional_property_item import TermsetAdditionalPropertyItem
from .update_account_pages import UpdateAccountPages
from .update_collection import UpdateCollection
from .update_collection_metadata import UpdateCollectionMetadata
from .update_project import UpdateProject
from .update_project_metadata import UpdateProjectMetadata
from .update_user import UpdateUser
from .update_user_metadata import UpdateUserMetadata
from .user import User
from .user_admin import UserAdmin
from .user_admin_update import UserAdminUpdate
from .user_get_users_response_200 import UserGetUsersResponse200
from .user_metadata import UserMetadata
from .visibility_level import VisibilityLevel

__all__ = (
    "Account",
    "AccountCreateAccountBody",
    "AccountCreateAccountBodyMetadata",
    "AccountList",
    "AccountListAccountsVisibilityLevel",
    "AccountMetadata",
    "AccountUpdateAccountBody",
    "AccountUpdateAccountBodyMetadata",
    "AdminGrantAdminAdminListRequest",
    "AdminGrants",
    "AdminList",
    "AuthActivateSessionBody",
    "AuthAuthenticateUserAuthenticationRequest",
    "AuthAuthenticateUserResponse200",
    "AuthGetCsrfResponse200",
    "AuthGetSessionResponse200",
    "AuthorizedActionsItem",
    "AuthProviderAuthorizedProvider",
    "AuthProviderLoginProvider",
    "AuthProviderSigninProviderProvider",
    "AuthProviderSigninProviderResponse200",
    "AuthProvidersReqResponse200",
    "AuthSignoutNoData",
    "Bundle",
    "BundleDefinition",
    "CacheFile",
    "CacheFileType",
    "CacheMem",
    "CacheMemType",
    "Cdeset",
    "CdesetList",
    "CdesListDataElementsResponse200",
    "CdesNewCdesetBody",
    "Collection",
    "CollectionAccessLevel",
    "CollectionAuthorization",
    "CollectionAuthorizationChange",
    "CollectionAuthorizationGrant",
    "CollectionAuthorizationInherited",
    "CollectionAuthorizationPermit",
    "CollectionAuthorizationRemoveIdentity",
    "CollectionAuthorizationRemoveIdentityAction",
    "CollectionAuthorizationSetIdentity",
    "CollectionAuthorizationSetIdentityAction",
    "CollectionAuthorizationUpdateAccessLevel",
    "CollectionAuthorizationUpdateAccessLevelAction",
    "CollectionCdeStats",
    "CollectionCdeStatsItemDistribItem",
    "CollectionCdeStatsPerCde",
    "CollectionMetadata",
    "CollectionMoveDatastore",
    "CollectionRole",
    "CollectionsDictionariesCopyToCdesetBody",
    "CollectionsDictionariesListBundlesResponse200",
    "CollectionsDictionariesListDataElementRefsResponse200",
    "CollectionsDictionariesListDataElementsResponse200",
    "CollectionsDictionariesListDictionariesResponse200",
    "CollectionsGetCollectionApcMembersFilter",
    "CollectionsGetCollectionAuthorizationFilter",
    "CollectionsGetCollectionsCollectionList",
    "CollectionsGetCollectionsSort",
    "CollectionsGetCollectionsSortDir",
    "CollectionsGetCollectionsVersion",
    "CollectionsItemsCompleteMultipartUploadBody",
    "CollectionsItemsCompleteMultipartUploadBodyAction",
    "CollectionsItemsCompleteMultipartUploadBodyChecksum",
    "CollectionsItemsCompleteMultipartUploadBodyPartsItem",
    "CollectionsItemsCopyItemBody",
    "CollectionsItemsCreateItemFilesFormStyleUpload",
    "CollectionsItemsCreateItemFilesFormStyleUploadMetadata",
    "CollectionsItemsCreateItemJsonNewItemRequest",
    "CollectionsItemsCreateItemJsonNewItemRequestContentType1",
    "CollectionsItemsGetItemDataDl",
    "CollectionsItemsInitiateMultipartUploadBody",
    "CollectionsItemsListItemsResponse200",
    "CollectionsItemsPutItemBody",
    "CollectionsItemsPutItemBodyContent",
    "CollectionsItemsPutItemBodyContentChecksum",
    "CollectionsTablesGetFormattedTableDataExportFormat",
    "CollectionsTablesGetTableDataElementsResponse200",
    "CollectionsTablesGetTableDataElementsResponse200ElementMap",
    "CollectionsTablesGetTableDataElementsResponse200ErrorMap",
    "CollectionsTablesListTablesResponse200",
    "CollectionsTablesPreviewTableDataBody",
    "CollectionStats",
    "CollectionStatsItemStats",
    "CollectionStatsItemStatsAdditionalProperty",
    "CollectionStatsItemStatsAdditionalPropertyNumFailed",
    "ColumnSchemaType",
    "DataDictionary",
    "DataDictionarySourceItem",
    "DataElement",
    "DataElementBundle",
    "DataElementConcept",
    "DataElementConceptAppliesTo",
    "DataElementDataType",
    "DataElementDefinition",
    "DataElementDefinitionDefinitionType",
    "DataElementPermissibleValuesDateTimeFormat",
    "DataElementPermissibleValuesDateTimeFormatDateTimeFormat",
    "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0",
    "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0Isoformat",
    "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1",
    "DataElementPermissibleValuesExternalReference",
    "DataElementPermissibleValuesExternalReferenceExternalReference",
    "DataElementPermissibleValuesNumberRange",
    "DataElementPermissibleValuesNumberRangeNumberRange",
    "DataElementPermissibleValuesTextRange",
    "DataElementPermissibleValuesTextRangeTextRange",
    "DataElementPermissibleValuesValueSet",
    "DataElementPermissibleValuesValueSetValueSetItem",
    "DataElementReference",
    "Datastore",
    "DatastoreType",
    "DictionarySearchOptions",
    "DictionarySearchOptionsOptions",
    "DictionarySearchOptionsOptionsAdditionalProperty",
    "DictionarySearchOptionsOptionsAdditionalPropertyType",
    "Error",
    "EventMetric",
    "EventMetricDimensions",
    "EventMetricType",
    "FederationActivity",
    "FederationAddressOrObjectType2",
    "FederationCollection",
    "FederationCollectionPage",
    "FederationCollectionPageType",
    "FederationCollectionType",
    "FederationUser",
    "FederationUserType",
    "Group",
    "GroupCreateGroupBody",
    "GroupCreateGroupBodyMetadata",
    "GroupGetGroupsResponse200",
    "GroupGetGroupsResponse200SelfRoles",
    "GroupGetGroupsVisibilityLevel",
    "GroupMember",
    "GroupMembers",
    "GroupMembersUsers",
    "GroupMetadata",
    "GroupRole",
    "GroupUpdateGroupBody",
    "GroupUpdateGroupBodyMetadata",
    "Highlights",
    "HighlightsMetadata",
    "Identity",
    "Item",
    "ItemColumn",
    "ItemContent",
    "ItemContentChecksum",
    "ItemMetadata",
    "ItemParser",
    "ItemParserOptions",
    "ItemSensitivityLabels",
    "ItemStatus",
    "ItemStatusDetail",
    "ItemStatusDetails",
    "ItemStatusValue",
    "ItemStorage",
    "ItemStorageChecksum",
    "ItemType",
    "MetadataField",
    "MetadataFieldGroup",
    "MetadataFieldGroupResource",
    "MetadataFieldGroups",
    "MetadataFieldGroupsDataElements",
    "MetadataFields",
    "MetadataFieldsDataElements",
    "MetadataGetFieldGroupsByResourceResourceType",
    "MetadataMetric",
    "MetadataMetricType",
    "MetadataSetFieldGroupsBody",
    "MetadataTags",
    "MetricAggFunc",
    "MetricData",
    "MetricEvent",
    "MetricEventList",
    "MetricList",
    "MetricLog",
    "MetricLogEntry",
    "MetricLogEntryDimensions",
    "MetricsExportEventDataFormat",
    "MetricsQueryEventDataPeriod",
    "MetricTimepoint",
    "MetricTimepointPartitions",
    "MultipartUploadDetails",
    "MultipartUploadDetailsTargetUrls",
    "NewCollection",
    "NewCollectionMetadata",
    "NewCollectionVersion",
    "NewCollectionVersionMetadata",
    "NewDataElement",
    "NewDataElementDataType",
    "NewDataElementRef",
    "NewProject",
    "NewProjectMetadata",
    "NewUser",
    "OauthProvider",
    "Operations",
    "OrderedDictionary",
    "Pagination",
    "Parser",
    "ParserOption",
    "ParserOptions",
    "ParserOptionType",
    "ParsersGetParsersResponse200",
    "ParsersResolvePrqlModulesResponse200",
    "PreferredOrder",
    "Project",
    "ProjectMetadata",
    "ProjectsGetProjectsProjectList",
    "PrqlModule",
    "QueryDataElementRequest",
    "QueryDataElements",
    "QueryDataElementsElementMap",
    "QueryDataElementsErrorMap",
    "QueryDataElementsFramesItem",
    "RootGetDatastoresResponse200",
    "SearchAccountsResponse",
    "SearchAccountsResponseHitsItem",
    "SearchAccountsResponseHitsItemProjectsItem",
    "SearchCollectionsResponse",
    "SearchCollectionsResponseHitsItem",
    "SearchCollectionsResponseHitsItemDataElementsItem",
    "SearchCollectionsResponseHitsItemItemsItem",
    "SearchDictionariesByItemColumnHits",
    "SearchDictionariesByItemColumnHitsSearchDictionariesHit",
    "SearchDictionariesByItemInverseResponse",
    "SearchDictionariesByItemResponse",
    "SearchDictionariesInverseResult",
    "SearchDictionariesInverseResultQueriesItem",
    "SearchDictionariesResponse",
    "SearchGetDictionarySearchOptionsResponse200",
    "SearchSearchAccountsBody",
    "SearchSearchAccountsBodyTypesItem",
    "SearchSearchCollectionsBody",
    "SearchSearchCollectionsBodyTypesItem",
    "SearchSearchDictionariesBody",
    "SearchSearchDictionariesBodyOptions",
    "SearchSearchDictionariesByItemBody",
    "SearchSearchDictionariesByItemBodyMethod",
    "SearchSearchDictionariesByItemInverseBody",
    "SearchSearchDictionariesByItemInverseBodyMethod",
    "ServerError",
    "SessionToken",
    "SessionUser",
    "SessionUserMetadata",
    "SystemConfiguration",
    "SystemConfigurationAuthentication",
    "SystemConfigurationAuthenticationCredentials",
    "SystemConfigurationAuthenticationCredentialsType",
    "SystemConfigurationAuthenticationGithub",
    "SystemConfigurationAuthenticationGithubType",
    "SystemConfigurationAuthenticationGoogle",
    "SystemConfigurationAuthenticationGoogleType",
    "SystemConfigurationAuthenticationOauth",
    "SystemConfigurationAuthenticationOauthType",
    "SystemConfigurationCache",
    "SystemConfigurationCacheCacheType",
    "SystemConfigurationDatastores",
    "SystemConfigurationDatastoresLocal",
    "SystemConfigurationDatastoresLocalType",
    "SystemConfigurationDatastoresPlugin",
    "SystemConfigurationDatastoresPluginOptions",
    "SystemConfigurationDatastoresPluginType",
    "SystemConfigurationDatastoresS3",
    "SystemConfigurationDatastoresS3Type",
    "SystemConfigurationPlugins",
    "SystemConfigurationPluginsAdditionalProperty",
    "SystemConfigurationSessions",
    "SystemConfigurationWorkers",
    "SystemConfigurationWorkersBackendType",
    "Table",
    "TableData",
    "TableDataDataItem",
    "TableDataDataModel",
    "TableDataModel",
    "TaskDef",
    "TaskDefDataType4",
    "TaskDefRetryItem",
    "TaskResult",
    "TaskResultDataType4",
    "TaskResultErrorsItem",
    "TaskResultStatus",
    "TaskScheduleDef",
    "TaskScheduleDefFrequency",
    "TasksResults",
    "TasksSchedules",
    "TasksSchedulesSchedulesItem",
    "TasksStats",
    "Termset",
    "TermsetAdditionalPropertyItem",
    "UpdateAccountPages",
    "UpdateCollection",
    "UpdateCollectionMetadata",
    "UpdateProject",
    "UpdateProjectMetadata",
    "UpdateUser",
    "UpdateUserMetadata",
    "User",
    "UserAdmin",
    "UserAdminUpdate",
    "UserGetUsersResponse200",
    "UserMetadata",
    "VisibilityLevel",
)



import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.group import Group
from ..models.user_metadata import UserMetadata
from ..models.visibility_level import VisibilityLevel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """User model

    Attributes:
        account_id (str):
        created_on (datetime.datetime):
        metadata (UserMetadata):
        subject_id (str):
        user_name (str):
        visibility_level (VisibilityLevel):
        given_name (Union[Unset, str]):
        groups (Union[Unset, List['Group']]): Names and ids of the groups this user belongs to
        last_name (Union[Unset, str]):
        user_email (Union[Unset, str]):
    """

    account_id: str
    created_on: datetime.datetime
    metadata: "UserMetadata"
    subject_id: str
    user_name: str
    visibility_level: VisibilityLevel
    given_name: Union[Unset, str] = UNSET
    groups: Union[Unset, List["Group"]] = UNSET
    last_name: Union[Unset, str] = UNSET
    user_email: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account_id = self.account_id
        created_on = self.created_on.isoformat()
        metadata = self.metadata.to_dict()
        subject_id = self.subject_id
        user_name = self.user_name
        visibility_level = self.visibility_level.value
        given_name = self.given_name
        groups: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.groups, Unset):
            groups = []
            for groups_item_data in self.groups:
                groups_item = groups_item_data.to_dict()
                groups.append(groups_item)

        last_name = self.last_name
        user_email = self.user_email

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "account_id": account_id,
                "created_on": created_on,
                "metadata": metadata,
                "subject_id": subject_id,
                "user_name": user_name,
                "visibility_level": visibility_level,
            }
        )
        if given_name is not UNSET:
            field_dict["given_name"] = given_name
        if groups is not UNSET:
            field_dict["groups"] = groups
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if user_email is not UNSET:
            field_dict["user_email"] = user_email

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`User` from a dict"""
        d = src_dict.copy()
        account_id = d.pop("account_id")

        created_on = isoparse(d.pop("created_on"))

        metadata = UserMetadata.from_dict(d.pop("metadata"))

        subject_id = d.pop("subject_id")

        user_name = d.pop("user_name")

        visibility_level = VisibilityLevel(d.pop("visibility_level"))

        given_name = d.pop("given_name", UNSET)

        groups = []
        _groups = d.pop("groups", UNSET)
        for groups_item_data in _groups or []:
            groups_item = Group.from_dict(groups_item_data)

            groups.append(groups_item)

        last_name = d.pop("last_name", UNSET)

        user_email = d.pop("user_email", UNSET)

        user = cls(
            account_id=account_id,
            created_on=created_on,
            metadata=metadata,
            subject_id=subject_id,
            user_name=user_name,
            visibility_level=visibility_level,
            given_name=given_name,
            groups=groups,
            last_name=last_name,
            user_email=user_email,
        )

        return user



"""Upload a part of a multipart item upload to a local datastore"""
from http import HTTPStatus
from typing import Any
from typing import cast
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.server_error import ServerError
from ...types import File
from ...types import Response


def _get_kwargs(
    datastore: str,
    upload_id: str,
    part_number: int,
    *,
    body: File,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": "/datastores/{datastore}/uploadPart/{upload_id}/{part_number}".format(
            datastore=datastore,
            upload_id=upload_id,
            part_number=part_number,
        ),
    }

    _body = body.payload

    _kwargs["content"] = _body
    headers["Content-Type"] = "application/octet-stream"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Any, ServerError]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        response_204 = cast(Any, {"attribute": "None", "return_type": "None"})
        return response_204
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    datastore: str,
    upload_id: str,
    part_number: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: File,
) -> Response[Union[Any, ServerError]]:
    """Upload a part of a multipart item upload to a local datastore

    For local (filesystem-based) datastores, this will be the
    endpoint that you can use to upload individual parts of a
    multipart item upload. Before sending parts to this endpoint,
    you will need to call `initiate_multipart_upload` for the
    desired collection and item ID; it will provide URLs to this
    endpoint.

    Args:
        datastore (str):
        upload_id (str):
        part_number (int):
        body (File):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ServerError]]
    """

    kwargs = _get_kwargs(
        datastore=datastore,
        upload_id=upload_id,
        part_number=part_number,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    datastore: str,
    upload_id: str,
    part_number: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: File,
) -> Union[Any]:
    """Upload a part of a multipart item upload to a local datastore

    For local (filesystem-based) datastores, this will be the
    endpoint that you can use to upload individual parts of a
    multipart item upload. Before sending parts to this endpoint,
    you will need to call `initiate_multipart_upload` for the
    desired collection and item ID; it will provide URLs to this
    endpoint.

    Args:
        datastore (str):
        upload_id (str):
        part_number (int):
        body (File):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any]
    """

    response = sync_detailed(
        datastore=datastore,
        upload_id=upload_id,
        part_number=part_number,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    datastore: str,
    upload_id: str,
    part_number: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: File,
) -> Response[Union[Any, ServerError]]:
    """Upload a part of a multipart item upload to a local datastore

    For local (filesystem-based) datastores, this will be the
    endpoint that you can use to upload individual parts of a
    multipart item upload. Before sending parts to this endpoint,
    you will need to call `initiate_multipart_upload` for the
    desired collection and item ID; it will provide URLs to this
    endpoint.

    Args:
        datastore (str):
        upload_id (str):
        part_number (int):
        body (File):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ServerError]]
    """

    kwargs = _get_kwargs(
        datastore=datastore,
        upload_id=upload_id,
        part_number=part_number,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    datastore: str,
    upload_id: str,
    part_number: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: File,
) -> Union[Any]:
    """Upload a part of a multipart item upload to a local datastore

    For local (filesystem-based) datastores, this will be the
    endpoint that you can use to upload individual parts of a
    multipart item upload. Before sending parts to this endpoint,
    you will need to call `initiate_multipart_upload` for the
    desired collection and item ID; it will provide URLs to this
    endpoint.

    Args:
        datastore (str):
        upload_id (str):
        part_number (int):
        body (File):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any]
    """

    response = await asyncio_detailed(
        datastore=datastore,
        upload_id=upload_id,
        part_number=part_number,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element import DataElement
from ..models.search_dictionaries_inverse_result_queries_item import (
    SearchDictionariesInverseResultQueriesItem,
)


T = TypeVar("T", bound="SearchDictionariesInverseResult")


@_attrs_define
class SearchDictionariesInverseResult:
    """SearchDictionariesInverseResult model

    Attributes:
        element (DataElement): Delfini Data Element
        queries (List['SearchDictionariesInverseResultQueriesItem']):
    """

    element: "DataElement"
    queries: List["SearchDictionariesInverseResultQueriesItem"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        element = self.element.to_dict()
        queries = []
        for queries_item_data in self.queries:
            queries_item = queries_item_data.to_dict()
            queries.append(queries_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "element": element,
                "queries": queries,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchDictionariesInverseResult` from a dict"""
        d = src_dict.copy()
        element = DataElement.from_dict(d.pop("element"))

        queries = []
        _queries = d.pop("queries")
        for queries_item_data in _queries:
            queries_item = SearchDictionariesInverseResultQueriesItem.from_dict(
                queries_item_data
            )

            queries.append(queries_item)

        search_dictionaries_inverse_result = cls(
            element=element,
            queries=queries,
        )

        return search_dictionaries_inverse_result



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.data_element_permissible_values_date_time_format_date_time_format_format_type_0 import (
    DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0,
)
from ..models.data_element_permissible_values_date_time_format_date_time_format_format_type_1 import (
    DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DataElementPermissibleValuesDateTimeFormatDateTimeFormat")


@_attrs_define
class DataElementPermissibleValuesDateTimeFormatDateTimeFormat:
    """Controls on the date, time or duration domain of the data.

    Attributes:
        format_ (Union['DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0',
            'DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1']):
        meaning (Union[Unset, str]): A description of this format's meaning.
    """

    format_: Union[
        "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0",
        "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1",
    ]
    meaning: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        format_: Dict[str, Any]
        if isinstance(
            self.format_,
            DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0,
        ):
            format_ = self.format_.to_dict()
        else:
            format_ = self.format_.to_dict()

        meaning = self.meaning

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "format": format_,
            }
        )
        if meaning is not UNSET:
            field_dict["meaning"] = meaning

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesDateTimeFormatDateTimeFormat` from a dict"""
        d = src_dict.copy()

        def _parse_format_(
            data: object,
        ) -> Union[
            "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0",
            "DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                format_type_0 = DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType0.from_dict(
                    data
                )

                return format_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            format_type_1 = DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1.from_dict(
                data
            )

            return format_type_1

        format_ = _parse_format_(d.pop("format"))

        meaning = d.pop("meaning", UNSET)

        data_element_permissible_values_date_time_format_date_time_format = cls(
            format_=format_,
            meaning=meaning,
        )

        return data_element_permissible_values_date_time_format_date_time_format



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.metric_timepoint_partitions import MetricTimepointPartitions
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="MetricTimepoint")


@_attrs_define
class MetricTimepoint:
    """MetricTimepoint model

    Attributes:
        dt (datetime.datetime):
        value (Union[float, int]):
        partitions (Union[Unset, MetricTimepointPartitions]):
    """

    dt: datetime.datetime
    value: Union[float, int]
    partitions: Union[Unset, "MetricTimepointPartitions"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        dt = self.dt.isoformat()
        value: Union[float, int]
        value = self.value
        partitions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.partitions, Unset):
            partitions = self.partitions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "dt": dt,
                "value": value,
            }
        )
        if partitions is not UNSET:
            field_dict["partitions"] = partitions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetricTimepoint` from a dict"""
        d = src_dict.copy()
        dt = isoparse(d.pop("dt"))

        def _parse_value(data: object) -> Union[float, int]:
            return cast(Union[float, int], data)

        value = _parse_value(d.pop("value"))

        _partitions = d.pop("partitions", UNSET)
        partitions: Union[Unset, MetricTimepointPartitions]
        if isinstance(_partitions, Unset):
            partitions = UNSET
        else:
            partitions = MetricTimepointPartitions.from_dict(_partitions)

        metric_timepoint = cls(
            dt=dt,
            value=value,
            partitions=partitions,
        )

        return metric_timepoint



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.data_element import DataElement
from ..models.data_element_bundle import DataElementBundle
from ..models.pagination import Pagination


T = TypeVar("T", bound="OrderedDictionary")


@_attrs_define
class OrderedDictionary:
    """OrderedDictionary model

    Attributes:
        entries (List[Union['DataElement', 'DataElementBundle']]):
        pagination (Pagination):
    """

    entries: List[Union["DataElement", "DataElementBundle"]]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        entries = []
        for entries_item_data in self.entries:
            entries_item: Dict[str, Any]
            if isinstance(entries_item_data, DataElement):
                entries_item = entries_item_data.to_dict()
            else:
                entries_item = entries_item_data.to_dict()

            entries.append(entries_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "entries": entries,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`OrderedDictionary` from a dict"""
        d = src_dict.copy()
        entries = []
        _entries = d.pop("entries")
        for entries_item_data in _entries:

            def _parse_entries_item(
                data: object,
            ) -> Union["DataElement", "DataElementBundle"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    entries_item_type_0 = DataElement.from_dict(data)

                    return entries_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                entries_item_type_1 = DataElementBundle.from_dict(data)

                return entries_item_type_1

            entries_item = _parse_entries_item(entries_item_data)

            entries.append(entries_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        ordered_dictionary = cls(
            entries=entries,
            pagination=pagination,
        )

        return ordered_dictionary



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.parser_option_type import ParserOptionType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="ParserOption")


@_attrs_define
class ParserOption:
    """ParserOption model

    Attributes:
        type (ParserOptionType):
        allow_null (Union[Unset, bool]):  Default: False.
        choices (Union[Unset, List[Any]]):
        default (Union[Unset, Any]):
        description (Union[Unset, str]):
    """

    type: ParserOptionType
    allow_null: Union[Unset, bool] = False
    choices: Union[Unset, List[Any]] = UNSET
    default: Union[Unset, Any] = UNSET
    description: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        type = self.type.value
        allow_null = self.allow_null
        choices: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.choices, Unset):
            choices = self.choices

        default = self.default
        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "type": type,
            }
        )
        if allow_null is not UNSET:
            field_dict["allow_null"] = allow_null
        if choices is not UNSET:
            field_dict["choices"] = choices
        if default is not UNSET:
            field_dict["default"] = default
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ParserOption` from a dict"""
        d = src_dict.copy()
        type = ParserOptionType(d.pop("type"))

        allow_null = d.pop("allow_null", UNSET)

        choices = cast(List[Any], d.pop("choices", UNSET))

        default = d.pop("default", UNSET)

        description = d.pop("description", UNSET)

        parser_option = cls(
            type=type,
            allow_null=allow_null,
            choices=choices,
            default=default,
            description=description,
        )

        return parser_option



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element_permissible_values_external_reference_external_reference import (
    DataElementPermissibleValuesExternalReferenceExternalReference,
)


T = TypeVar("T", bound="DataElementPermissibleValuesExternalReference")


@_attrs_define
class DataElementPermissibleValuesExternalReference:
    """DataElementPermissibleValuesExternalReference model

    Attributes:
        external_reference (DataElementPermissibleValuesExternalReferenceExternalReference): A reference to an external
            source of permissible values.
    """

    external_reference: "DataElementPermissibleValuesExternalReferenceExternalReference"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        external_reference = self.external_reference.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "externalReference": external_reference,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesExternalReference` from a dict"""
        d = src_dict.copy()
        external_reference = (
            DataElementPermissibleValuesExternalReferenceExternalReference.from_dict(
                d.pop("externalReference")
            )
        )

        data_element_permissible_values_external_reference = cls(
            external_reference=external_reference,
        )

        return data_element_permissible_values_external_reference



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.metric_log_entry_dimensions import MetricLogEntryDimensions


T = TypeVar("T", bound="MetricLogEntry")


@_attrs_define
class MetricLogEntry:
    """MetricLogEntry model

    Attributes:
        dimensions (MetricLogEntryDimensions):
        dt (datetime.datetime):
        name (str):
        value (Union[float, int]):
    """

    dimensions: "MetricLogEntryDimensions"
    dt: datetime.datetime
    name: str
    value: Union[float, int]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        dimensions = self.dimensions.to_dict()
        dt = self.dt.isoformat()
        name = self.name
        value: Union[float, int]
        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "dimensions": dimensions,
                "dt": dt,
                "name": name,
                "value": value,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetricLogEntry` from a dict"""
        d = src_dict.copy()
        dimensions = MetricLogEntryDimensions.from_dict(d.pop("dimensions"))

        dt = isoparse(d.pop("dt"))

        name = d.pop("name")

        def _parse_value(data: object) -> Union[float, int]:
            return cast(Union[float, int], data)

        value = _parse_value(d.pop("value"))

        metric_log_entry = cls(
            dimensions=dimensions,
            dt=dt,
            name=name,
            value=value,
        )

        return metric_log_entry



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="MetadataField")


@_attrs_define
class MetadataField:
    """MetadataField model

    Attributes:
        name (str):
        data_element_ref (Union[Unset, str]):
        is_array (Union[Unset, bool]):  Default: False.
        label (Union[Unset, str]):
    """

    name: str
    data_element_ref: Union[Unset, str] = UNSET
    is_array: Union[Unset, bool] = False
    label: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        name = self.name
        data_element_ref = self.data_element_ref
        is_array = self.is_array
        label = self.label

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if data_element_ref is not UNSET:
            field_dict["data_element_ref"] = data_element_ref
        if is_array is not UNSET:
            field_dict["is_array"] = is_array
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetadataField` from a dict"""
        d = src_dict.copy()
        name = d.pop("name")

        data_element_ref = d.pop("data_element_ref", UNSET)

        is_array = d.pop("is_array", UNSET)

        label = d.pop("label", UNSET)

        metadata_field = cls(
            name=name,
            data_element_ref=data_element_ref,
            is_array=is_array,
            label=label,
        )

        return metadata_field
